@@ -16,6 +16,7 @@ from .compl import ComplOID, ComplSvc, ComplNode, ComplYamlFile
 from .compl import ComplOIDtp, ComplSvcRpcMethod, ComplSvcRpcParams, ComplEdit
 from .client import call_rpc, DEFAULT_DB_SERVICE, DEFAULT_REPL_SERVICE
 from .client import DEFAULT_ACL_SERVICE, DEFAULT_AUTH_SERVICE
+from .client import DEFAULT_KIOSK_SERVICE
 
 DEFAULT_RPC_ERROR_MESSAGE = {
     -32700: 'parse error',
@@ -938,6 +939,184 @@ def sys_cmd(cmd):
         os.system(cmd)
 
 
+def append_kiosk_cli(root_sp):
+    ap = root_sp.add_parser('kiosk', help='kiosk commands')
+    sp = ap.add_subparsers(dest='_subc', help='sub command')
+
+    p = sp.add_parser('login', help='login kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('logout', help='logout kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('test', help='test kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('info', help='kiosk info')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('alert', help='display alert on kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument('text', metavar='TEXT')
+    p.add_argument('-l', '--level', choices=['info', 'warning'])
+    p.add_argument('--timeout', metavar='SEC', dest='t', type=int)
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('eval', help='execute JavaScript code on kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument('code', metavar='CODE')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('navigate', help='navigate kiosk to a page')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument('url', metavar='URL', nargs='?')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('display', help='control kiosk display')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument('--off', action='store_true')
+    p.add_argument('--on', action='store_true')
+    p.add_argument('--brightness', metavar='LEVEL', type=int)
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('zoom', help='change kiosk zoom level')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument('level', metavar='LEVEL', type=float)
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('reload', help='reload kiosk browser')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('reboot', help='reboot kiosk OS')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('dev_open', help='open kiosk development console')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('dev_close', help='close kiosk development console')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('list', help='list kiosks')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('edit', help='edit kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('export', help='export kiosks(s) to a deployment file')
+    p.add_argument('i', metavar='MASK')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+    p.add_argument('-o', '--output', metavar='FILE',
+                   help='output file').completer = ComplYamlFile()
+
+    p = sp.add_parser('deploy', help='deploy kiosk(s) from a deployment file')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+    p.add_argument('-f', '--file', metavar='FILE',
+                   help='deployment file').completer = ComplYamlFile()
+
+    p = sp.add_parser('undeploy',
+                      help='undeploy kiosk(s) using a deployment file')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+    p.add_argument('-f', '--file', metavar='FILE',
+                   help='deployment file').completer = ComplYamlFile()
+
+    p = sp.add_parser('create', help='create kiosk')
+    p.add_argument('i', metavar='kiosk', help='kiosk id')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+    p = sp.add_parser('destroy', help='destroy kiosk')
+    p.add_argument('i', metavar='kiosk')
+    p.add_argument(
+        '-a',
+        '--kiosk-svc',
+        help=f'kiosk service (default: {DEFAULT_KIOSK_SERVICE})',
+        default=DEFAULT_KIOSK_SERVICE).completer = ComplSvc('kioskman')
+
+
 def init_ap():
     ap = Parser()
 
@@ -995,6 +1174,7 @@ def init_ap():
     append_cloud_cli(sp)
     append_system_cli(sp)
     append_mirror_cli(sp)
+    append_kiosk_cli(sp)
 
     sp.add_parser('save', help='save scheduled states (if instant-save is off)')
     append_svc_cli(sp)
