@@ -3,6 +3,7 @@ import sys
 import time
 from datetime import datetime
 from functools import partial
+from collections import OrderedDict
 
 import elbus
 from evaics.sdk import pack
@@ -13,7 +14,7 @@ from .tools import print_result, ok, edit_config, read_file, write_file
 from .tools import print_action_result, format_value, prepare_time, err, warn
 from .tools import get_node_svc_info, edit_file, get_term_size, exec_cmd, xc
 from .tools import get_my_ip
-from .client import call_rpc, DEFAULT_REPL_SERVICE
+from .client import call_rpc, DEFAULT_REPL_SERVICE, connect
 from .sharedobj import common, current_command
 
 from . import DEFAULT_REPOSITORY_URL
@@ -708,15 +709,15 @@ class CLI:
                 t = r.get('time', {}).get('accepted')
                 if t:
                     t = datetime.fromtimestamp(t, common.TZ).isoformat()
-                data.append({
-                    'time': t,
-                    'uuid': str(uuid.UUID(bytes=r['uuid'])),
-                    'oid': r['oid'],
-                    'status': r['status'],
-                    'elapsed': elapsed,
-                    'node': r['node'],
-                    'svc': r['svc']
-                })
+                d = OrderedDict()
+                d['time'] = t
+                d['uuid'] = str(uuid.UUID(bytes=r['uuid']))
+                d['oid'] = r['oid']
+                d['status'] = r['status']
+                d['elapsed'] = elapsed
+                d['node'] = r['node']
+                d['svc'] = r['svc']
+                data.append(d)
             spacer = '  '
             header, rows = format_table(data, fmt=FORMAT_GENERATOR_COLS)
             for i, c in enumerate(header):
@@ -732,7 +733,7 @@ class CLI:
                 for i, c in enumerate(r):
                     if i > 0:
                         print(spacer, end='')
-                    if i == 2:
+                    if i == 3:
                         print(colored(c,
                                       color=ACTION_STATUS_COLOR.get(
                                           d['status'])),
@@ -890,12 +891,17 @@ class CLI:
         if value is not None:
             val = format_value(value)
             payload['value'] = val
+        connect()
         common.bus.send(
             'RAW/' + i.replace(':', '/', 1),
             elbus.client.Frame(pack(payload), tp=elbus.client.OP_PUBLISH,
                                qos=1)).wait_completed()
         time.sleep(0.1)
-        state = call_rpc('item.state', dict(i=i))[0]
+        try:
+            state = call_rpc('item.state', dict(i=i))[0]
+        except IndexError:
+            err('no such item')
+            return
         try:
             if state['status'] != status or (value is not None and
                                              state['value'] != val):
@@ -1266,3 +1272,169 @@ class CLI:
               'create it with "eva mirror update" command.')
         print('If the mirror service is set to different port, '
               'replace it in the URLs above.')
+
+    def kiosk_list(self, kiosk_svc):
+        data = call_rpc('kiosk.list', target=kiosk_svc)
+        if current_command.json:
+            print_result(data)
+        else:
+            formatted_data = []
+            for d in data:
+                state = d.get('state')
+                info = OrderedDict()
+                info['name'] = d.get('name')
+                info['ip'] = d.get('ip')
+                info['a.log'] = d.get('auto_login')
+                info['agent'] = d.get('agent')
+                info['version'] = d.get('version')
+                info['state'] = state if state else 'offline'
+                info['current url'] = d.get('current_url')
+                formatted_data.append(info)
+            spacer = '  '
+            header, rows = format_table(formatted_data,
+                                        fmt=FORMAT_GENERATOR_COLS)
+            for i, c in enumerate(header):
+                if i > 0:
+                    print(spacer, end='')
+                print(colored(c, color='blue'), end='')
+            print()
+            print(
+                colored('-' * (sum(len(s) for s in header) +
+                               (len(header) - 1) * 2),
+                        color='grey'))
+            for d, r in zip(formatted_data, rows):
+                for i, c in enumerate(r):
+                    if i > 0:
+                        print(spacer, end='')
+                    if i == 5:
+                        if d['state'] == 'active':
+                            color = 'green'
+                        elif d['state'] == 'offline':
+                            color = 'grey'
+                        else:
+                            color = None
+                        print(colored(c, color=color), end='')
+                    else:
+                        print(c, end='')
+                print()
+
+    def kiosk_edit(self, i, kiosk_svc):
+
+        def deploy(cfg, i):
+            call_rpc('kiosk.deploy', dict(kiosks=[cfg]), target=kiosk_svc)
+            print(f'Kiosk configuration re-deployed: {i}')
+            print()
+
+        config = call_rpc('kiosk.get_config', dict(i=i), target=kiosk_svc)
+        edit_config(config, f'kiosk|{i}|config', deploy_fn=partial(deploy, i=i))
+
+    def kiosk_export(self, i, kiosk_svc, output=None):
+        c = 0
+        configs = []
+        for kiosk in call_rpc('kiosk.list', target=kiosk_svc):
+            name = kiosk['name']
+            if (i.startswith('*') and name.endswith(i[1:])) or \
+                (i.endswith('*') and name.startswith(i[:-1])) or \
+                (i.startswith('*') and i.endswith('*') and i[1:-1] in name) or \
+                i == name:
+                c += 1
+                config = call_rpc('kiosk.get_config',
+                                  dict(i=name),
+                                  target=kiosk_svc)
+                configs.append(config)
+        result = dict(kiosks=configs)
+        if current_command.json:
+            print_result(result)
+        else:
+            import yaml
+            dump = yaml.dump(result, default_flow_style=False)
+            if output is None:
+                print(dump)
+            else:
+                write_file(output, dump)
+                print(f'{c} kiosk configuration(s) exported')
+                print()
+
+    def kiosk_deploy(self, kiosk_svc, file=None):
+        import yaml
+        kiosks = yaml.safe_load(read_file(file).decode()).pop('kiosks')
+        call_rpc('kiosk.deploy', dict(kiosks=kiosks), target=kiosk_svc)
+        print(f'{len(kiosks)} kiosk configuration(s) deployed')
+        print()
+
+    def kiosk_undeploy(self, kiosk_svc, file=None):
+        import yaml
+        kiosks = yaml.safe_load(read_file(file).decode()).pop('kiosks')
+        call_rpc('kiosk.undeploy', dict(kiosks=kiosks), target=kiosk_svc)
+        print(f'{len(kiosks)} kiosk configuration(s) undeployed')
+        print()
+
+    def kiosk_create(self, i, kiosk_svc):
+        call_rpc('kiosk.deploy', dict(kiosks=[dict(name=i)]), target=kiosk_svc)
+        ok()
+
+    def kiosk_destroy(self, i, kiosk_svc):
+        call_rpc('kiosk.destroy', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_login(self, i, kiosk_svc):
+        call_rpc('kiosk.login', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_logout(self, i, kiosk_svc):
+        call_rpc('kiosk.logout', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_test(self, i, kiosk_svc):
+        call_rpc('kiosk.test', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_info(self, i, kiosk_svc):
+        result = call_rpc('kiosk.info', dict(i=i), target=kiosk_svc)
+        print_result(result, name_value=True)
+
+    def kiosk_alert(self, i, kiosk_svc, text, level=None, t=None):
+        call_rpc('kiosk.alert',
+                 dict(i=i, text=text, level=level, timeout=t),
+                 target=kiosk_svc)
+        ok()
+
+    def kiosk_eval(self, i, code, kiosk_svc):
+        call_rpc('kiosk.eval', dict(i=i, code=code), target=kiosk_svc)
+        ok()
+
+    def kiosk_navigate(self, i, url, kiosk_svc):
+        call_rpc('kiosk.navigate', dict(i=i, url=url), target=kiosk_svc)
+        ok()
+
+    def kiosk_display(self, i, kiosk_svc, off, on, brightness=None):
+        if on:
+            dst = True
+        elif off:
+            dst = False
+        else:
+            dst = None
+        call_rpc('kiosk.display',
+                 dict(i=i, on=dst, brightness=brightness),
+                 target=kiosk_svc)
+        ok()
+
+    def kiosk_zoom(self, i, level, kiosk_svc):
+        call_rpc('kiosk.zoom', dict(i=i, level=level), target=kiosk_svc)
+        ok()
+
+    def kiosk_reload(self, i, kiosk_svc):
+        call_rpc('kiosk.reload', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_reboot(self, i, kiosk_svc):
+        call_rpc('kiosk.reboot', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_dev_open(self, i, kiosk_svc):
+        call_rpc('kiosk.dev_open', dict(i=i), target=kiosk_svc)
+        ok()
+
+    def kiosk_dev_close(self, i, kiosk_svc):
+        call_rpc('kiosk.dev_close', dict(i=i), target=kiosk_svc)
+        ok()
