@@ -14,7 +14,10 @@ from .sharedobj import current_command, common
 
 
 def ok():
-    print(colored('OK', color='green'))
+    if current_command.json:
+        print('null')
+    else:
+        print(colored('OK', color='green'))
 
 
 def debug(s):
@@ -441,8 +444,8 @@ def get_term_size():
 
 def get_node_svc_info():
     import json
-    return json.loads(
-        os.popen(f'{common.dir_eva}/svc/eva-node --mode info').read())
+    with os.popen(f'{common.dir_eva}/svc/eva-node --mode info') as p:
+        return json.loads(p.read())
 
 
 def get_arch():
